from __future__ import absolute_import

import numpy as np
import os
import pytest

from mirdata import rwc_popular, utils
from tests.test_utils import DEFAULT_DATA_HOME
from tests.test_download_utils import mock_downloader


def test_track():
    # test data home None
    track_default = rwc_popular.Track('RM-P001')
    assert track_default._data_home == os.path.join(DEFAULT_DATA_HOME, 'RWC-Popular')

    # test data_home where the test data lives
    data_home = 'tests/resources/mir_datasets/RWC-Popular'

    with pytest.raises(ValueError):
        rwc_popular.Track('asdfasdf', data_home=data_home)

    track = rwc_popular.Track('RM-P001', data_home=data_home)

    # test attributes are loaded as expected
    assert track.track_id == 'RM-P001'
    assert track._data_home == data_home
    assert track._track_paths == {
        'audio': ['audio/rwc-p-m01/1.wav', '110ac7edb20dbe9a75ffe81b2bfeecef'],
        'sections': [
            'annotations/AIST.RWC-MDB-P-2001.CHORUS/RM-P001.CHORUS.TXT',
            '2d735867d44c4f8677b48746b5eb324d',
        ],
        'beats': [
            'annotations/AIST.RWC-MDB-P-2001.BEAT/RM-P001.BEAT.TXT',
            '523231aebfea1cc62bad575cda3f704b',
        ],
        'chords': [
            'annotations/AIST.RWC-MDB-P-2001.CHORD/RWC_Pop_Chords/N001-M01-T01.lab',
            '2a8b1d320bb88f710be3bff4339db99b',
        ],
        'voca_inst': [
            'annotations/AIST.RWC-MDB-P-2001.VOCA_INST/RM-P001.VOCA_INST.TXT',
            'f3ee36598a8bb9e367d25e0ff99850c7',
        ],
    }
    assert (
        track.audio_path
        == 'tests/resources/mir_datasets/RWC-Popular/' + 'audio/rwc-p-m01/1.wav'
    )
    assert track.piece_number == 'No. 1'
    assert track.suffix == 'M01'
    assert track.track_number == 'Tr. 01'
    assert track.title == 'Eien no replica'
    assert track.artist == 'Kazuo Nishi'
    assert track.singer_information == 'Male'
    assert track.duration_sec == '03:29'
    assert track.tempo == '135'
    assert track.instruments == 'Gt'
    assert track.drum_information == 'Drum sequences'

    # test that cached properties don't fail and have the expected type
    assert type(track.sections) is utils.SectionData
    assert type(track.beats) is utils.BeatData
    assert type(track.chords) is utils.ChordData
    assert type(track.vocal_instrument_activity) is utils.EventData

    # test audio loading functions
    y, sr = track.audio
    assert sr == 44100
    assert y.shape == (44100 * 2,)

    repr_string = (
        "RWC-Popular Track(track_id=RM-P001, "
        + "audio_path=tests/resources/mir_datasets/RWC-Popular/audio/rwc-p-m01/1.wav, "
        + "piece_number=No. 1, suffix=M01, track_number=Tr. 01, title=Eien no replica, "
        + "artist=Kazuo Nishi, singer_information=Male, duration_sec=03:29, "
        + "tempo=135, instruments=Gt, drum_information=Drum sequences, "
        + "sections=SectionData('start_times', 'end_times', 'sections'), "
        + "beats=BeatData('beat_times', 'beat_positions'))"
    )
    assert track.__repr__() == repr_string


def test_track_ids():
    track_ids = rwc_popular.track_ids()
    assert type(track_ids) is list
    assert len(track_ids) == 100


def test_load():
    data_home = 'tests/resources/mir_datasets/RWC-Popular'
    rwc_popular_data = rwc_popular.load(data_home=data_home)
    assert type(rwc_popular_data) is dict
    assert len(rwc_popular_data.keys()) == 100

    rwc_popular_data_default = rwc_popular.load()
    assert type(rwc_popular_data_default) is dict
    assert len(rwc_popular_data_default.keys()) == 100


def test_load_chords():
    chords_path = (
        'tests/resources/mir_datasets/RWC-Popular/'
        + 'annotations/AIST.RWC-MDB-P-2001.CHORD/RWC_Pop_Chords/N001-M01-T01.lab'
    )
    chord_data = rwc_popular._load_chords(chords_path)

    # check types
    assert type(chord_data) is utils.ChordData
    assert type(chord_data.start_times) is np.ndarray
    assert type(chord_data.end_times) is np.ndarray
    assert type(chord_data.chords) is np.ndarray

    # check values
    assert np.array_equal(
        chord_data.start_times, np.array([0.000, 0.104, 3.646, 43.992, 44.494])
    )
    assert np.array_equal(
        chord_data.end_times, np.array([0.104, 1.858, 5.387, 44.494, 47.636])
    )
    assert np.array_equal(
        chord_data.chords, np.array(['N', 'Ab:min', 'E:maj', 'Bb:maj(*3)', 'C:min7'])
    )

    # load a file which doesn't exist
    chord_data_none = rwc_popular._load_chords('fake/path')
    assert chord_data_none is None


def test_load_voca_inst():
    vocinst_path = (
        'tests/resources/mir_datasets/RWC-Popular/'
        + 'annotations/AIST.RWC-MDB-P-2001.VOCA_INST/RM-P001.VOCA_INST.TXT'
    )
    vocinst_data = rwc_popular._load_voca_inst(vocinst_path)

    # check types
    assert type(vocinst_data) is utils.EventData
    assert type(vocinst_data.start_times) is np.ndarray
    assert type(vocinst_data.end_times) is np.ndarray
    assert type(vocinst_data.event) is np.ndarray

    # check values
    assert np.array_equal(
        vocinst_data.start_times,
        np.array(
            [
                0.000,
                10.293061224,
                11.883492063,
                12.087845804,
                13.587460317,
                13.819387755,
                20.668707482,
                20.832653061,
            ]
        ),
    )
    assert np.array_equal(
        vocinst_data.end_times,
        np.array(
            [
                10.293061224,
                11.883492063,
                12.087845804,
                13.587460317,
                13.819387755,
                20.668707482,
                20.832653061,
                26.465306122,
            ]
        ),
    )
    assert np.array_equal(
        vocinst_data.event,
        np.array(
            ['b', 'm:withm', 'b', 'm:withm', 'b', 'm:withm', 'b', 's:electricguitar']
        ),
    )

    # load a file which doesn't exist
    vocainst_data_none = rwc_popular._load_voca_inst('fake/path')
    assert vocainst_data_none is None


def test_load_metadata():
    data_home = 'tests/resources/mir_datasets/RWC-Popular'
    metadata = rwc_popular._load_metadata(data_home)
    assert metadata['data_home'] == data_home
    assert metadata['RM-P001'] == {
        'piece_number': 'No. 1',
        'suffix': 'M01',
        'track_number': 'Tr. 01',
        'title': 'Eien no replica',
        'artist': 'Kazuo Nishi',
        'singer_information': 'Male',
        'duration_sec': '03:29',
        'tempo': '135',
        'instruments': 'Gt',
        'drum_information': 'Drum sequences',
    }

    metadata_none = rwc_popular._load_metadata('asdf/asdf')
    assert metadata_none is None


def test_download(mock_downloader):
    rwc_popular.download()
    mock_downloader.assert_called()


def test_validate():
    rwc_popular.validate()
    rwc_popular.validate(silence=True)


def test_cite():
    rwc_popular.cite()



# -*- coding: utf-8 -*-
"""SALAMI Dataset Loader

SALAMI Dataset.
Details can be found at http://ddmal.music.mcgill.ca/research/salami/annotations


Attributes:
    DIR (str): The directory name for SALAMI dataset. Set to `'Salami'`.

    DATA.index (dict): {track_id: track_data}.
        track_data is a jason data loaded from `index/`

    DATA.metadata (None): (todo?)

    ANNOT_REMOTE (RemoteFileMetadata (namedtuple)): metadata
        of SALAMI dataset. It includes the annotation file name, annotation
        file url, and checksum of the file.


"""
import csv
import librosa
import logging
import numpy as np
import os

import mirdata.utils as utils
import mirdata.download_utils as download_utils

DATASET_DIR = 'Salami'
ANNOTATIONS_REMOTE = download_utils.RemoteFileMetadata(
    filename='salami-data-public-master.zip',
    url='https://github.com/DDMAL/salami-data-public/archive/master.zip',
    checksum='f88b3455f1f2443458d094f603c0c1c7',
    destination_dir=None,
)


def _load_metadata(data_home):

    metadata_path = os.path.join(
        data_home, os.path.join('salami-data-public-master', 'metadata', 'metadata.csv')
    )

    if not os.path.exists(metadata_path):
        logging.info('Metadata file {} not found.'.format(metadata_path))
        return None

    with open(metadata_path, 'r') as fhandle:
        reader = csv.reader(fhandle, delimiter=',')
        raw_data = []
        for line in reader:
            if line[0] == 'SONG ID':
                continue
            raw_data.append(line)

    metadata_index = {}
    for line in raw_data:
        track_id = line[0]

        metadata_index[track_id] = {
            'source': line[1],
            'annotator_1_id': line[2],
            'annotator_2_id': line[3],
            'duration_sec': line[5],
            'title': line[7],
            'artist': line[8],
            'annotator_1_time': line[10],
            'annotator_2_time': line[11],
            'class': line[14],
            'genre': line[15],
        }

    metadata_index['data_home'] = data_home

    return metadata_index


DATA = utils.LargeData('salami_index.json', _load_metadata)


class Track(object):
    """SALAMI Track class

    Args:
        track_id (str): track id of the track
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Attributes:
        track_id (str): track id
        audio_path (str): audio path of the track
        source
        annotator_1_id
        annotator_2_id
        duration_sec
        title
        artist
        annotator_1_time
        annotator_2-time
        broad_genre
        genre: (todo: somewhat separate metadata and annotation)
        sections_annotator_1_uppercase: annotation
        sections_annotator_1_lowercase: annotation
        sections_annotator_2_uppercase: annotation
        sections_annotator_2_lowercase: annotation


    """

    def __init__(self, track_id, data_home=None):
        if track_id not in DATA.index:
            raise ValueError('{} is not a valid track ID in Salami'.format(track_id))

        self.track_id = track_id

        if data_home is None:
            data_home = utils.get_default_dataset_path(DATASET_DIR)

        self._data_home = data_home
        self._track_paths = DATA.index[track_id]

        metadata = DATA.metadata(data_home)
        if metadata is not None and track_id in metadata:
            self._track_metadata = metadata[track_id]
        else:
            # annotations with missing metadata
            self._track_metadata = {
                'source': None,
                'annotator_1_id': None,
                'annotator_2_id': None,
                'duration_sec': None,
                'title': None,
                'artist': None,
                'annotator_1_time': None,
                'annotator_2_time': None,
                'class': None,
                'genre': None,
            }

        self.audio_path = os.path.join(self._data_home, self._track_paths['audio'][0])

        self.source = self._track_metadata['source']
        self.annotator_1_id = self._track_metadata['annotator_1_id']
        self.annotator_2_id = self._track_metadata['annotator_2_id']
        self.duration_sec = self._track_metadata['duration_sec']
        self.title = self._track_metadata['title']
        self.artist = self._track_metadata['artist']
        self.annotator_1_time = self._track_metadata['annotator_1_time']
        self.annotator_2_time = self._track_metadata['annotator_2_time']
        self.broad_genre = self._track_metadata['class']
        self.genre = self._track_metadata['genre']

    def __repr__(self):
        repr_string = (
            "Salami Track(track_id={}, audio_path={}, source={}, "
            + "title={}, artist={}, duration_sec={}, annotator_1_id={}, "
            + "annotator_2_id={}, annotator_1_time={}, annotator_2_time={}, "
            + "broad_genre={}, genre={}, "
            + "sections_annotator_1_uppercase=SectionData('start_times', 'end_times', 'sections'), "
            + "sections_annotator_1_lowercase=SectionData('start_times', 'end_times', 'sections'), "
            + "sections_annotator_2_uppercase=SectionData('start_times', 'end_times', 'sections'), "
            + "sections_annotator_2_lowercase=SectionData('start_times', 'end_times', 'sections')"
        )
        return repr_string.format(
            self.track_id,
            self.audio_path,
            self.source,
            self.title,
            self.artist,
            self.duration_sec,
            self.annotator_1_id,
            self.annotator_2_id,
            self.annotator_1_time,
            self.annotator_2_time,
            self.broad_genre,
            self.genre,
        )

    @utils.cached_property
    def sections_annotator_1_uppercase(self):
        if self._track_paths['annotator_1_uppercase'][0] is None:
            return None
        return _load_sections(
            os.path.join(self._data_home, self._track_paths['annotator_1_uppercase'][0])
        )

    @utils.cached_property
    def sections_annotator_1_lowercase(self):
        if self._track_paths['annotator_1_lowercase'][0] is None:
            return None
        return _load_sections(
            os.path.join(self._data_home, self._track_paths['annotator_1_lowercase'][0])
        )

    @utils.cached_property
    def sections_annotator_2_uppercase(self):
        if self._track_paths['annotator_2_uppercase'][0] is None:
            return None
        return _load_sections(
            os.path.join(self._data_home, self._track_paths['annotator_2_uppercase'][0])
        )

    @utils.cached_property
    def sections_annotator_2_lowercase(self):
        if self._track_paths['annotator_2_lowercase'][0] is None:
            return None
        return _load_sections(
            os.path.join(self._data_home, self._track_paths['annotator_2_lowercase'][0])
        )

    @property
    def audio(self):
        return librosa.load(self.audio_path, sr=None, mono=True)


def download(data_home=None, force_overwrite=False):
    """Download SALAMI Dataset (annotations).
    The audio files are not provided.

    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

        force_overwrite (bool): whether to overwrite the existing downloaded data

    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    info_message = """
        Unfortunately the audio files of the Salami dataset are not available
        for download. If you have the Salami dataset, place the contents into a
        folder called Salami with the following structure:
            > Salami/
                > salami-data-public-master/
                > audio/
        and copy the Salami folder to {}
    """.format(
        data_home
    )

    download_utils.downloader(
        data_home,
        zip_downloads=[ANNOTATIONS_REMOTE],
        info_message=info_message,
        force_overwrite=force_overwrite,
    )


def validate(data_home=None, silence=False):
    """Validate if the stored dataset is a valid version

    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Returns:
        missing_files (list): List of file paths that are in the dataset index
            but missing locally
        invalid_checksums (list): List of file paths that file exists in the dataset
            index but has a different checksum compare to the reference checksum

    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    missing_files, invalid_checksums = utils.validator(
        DATA.index, data_home, silence=silence
    )
    return missing_files, invalid_checksums


def track_ids():
    """Return track ids

    Returns:
        (list): A list of track ids
    """
    return list(DATA.index.keys())


def load(data_home=None):
    """Load SALAMI dataset

    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Returns:
        (dict): {`track_id`: track data}

    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    salami_data = {}
    for key in track_ids():
        salami_data[key] = Track(key, data_home=data_home)
    return salami_data


def _load_sections(sections_path):
    if sections_path is None or not os.path.exists(sections_path):
        return None

    times = []
    secs = []
    with open(sections_path, 'r') as fhandle:
        reader = csv.reader(fhandle, delimiter='\t')
        for line in reader:
            times.append(float(line[0]))
            secs.append(line[1])
    times = np.array(times)
    secs = np.array(secs)

    # remove sections with length == 0
    times_revised = np.delete(times, np.where(np.diff(times) == 0))
    secs_revised = np.delete(secs, np.where(np.diff(times) == 0))
    return utils.SectionData(
        np.array(times_revised[:-1]),
        np.array(times_revised)[1:],
        np.array(secs_revised)[:-1],
    )


def cite():
    """Print the reference"""

    cite_data = """
===========  MLA ===========
Smith, Jordan Bennett Louis, et al.,
"Design and creation of a large-scale database of structural annotations",
12th International Society for Music Information Retrieval Conference (2011)

========== Bibtex ==========
@inproceedings{smith2011salami,
    title={Design and creation of a large-scale database of structural annotations.},
    author={Smith, Jordan Bennett Louis and Burgoyne, John Ashley and
          Fujinaga, Ichiro and De Roure, David and Downie, J Stephen},
    booktitle={12th International Society for Music Information Retrieval Conference},
    year={2011},
    series = {ISMIR},
}
"""

    print(cite_data)



# -*- coding: utf-8 -*-
"""GuitarSet Loader

Intr​oducing GuitarSet, a dataset that provides high quality guitar
recordings alongside rich annotations and metadata.
In particular, by recording guitars using a hexaphonic pickup, we
are able to not only provide recordings of the individual strings
but also to largely automate the expensive annotation process,
therefore providing rich annotation.

The dataset contains recordings of a variety of musical excerpts
played on an acoustic guitar, along with time-aligned annotations
including pitch contours, string and fret positions, chords, beats,
downbeats, and keys.

Details can be found at http://github.com/marl/guitarset/

Attributes:
    DATASET_DIR (str):
        The directory name for GuitarSet. Set to `'GuitarSet'`.

    DATA.index (dict): {track_id: track_data}.
        track_data is a `GuitarSet` namedtuple.

    ANNOTATION_REMOTE (RemoteFileMetadata)
    AUDIO_HEX_CLN_REMOTE (RemoteFileMetadata)
    AUDIO_HEX_REMOTE (RemoteFileMetadata)
    AUDIO_MIC_REMOTE (RemoteFileMetadata)
    AUDIO_MIX_REMOTE (RemoteFileMetadata)
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import numpy as np
import os
import librosa
import jams
import logging

import mirdata.utils as utils
import mirdata.download_utils as download_utils

DATASET_DIR = 'GuitarSet'

ANNOTATION_REMOTE = download_utils.RemoteFileMetadata(
    filename='annotation.zip',
    url='https://zenodo.org/record/3371780/files/annotation.zip?download=1',
    checksum='b39b78e63d3446f2e54ddb7a54df9b10',
    destination_dir='annotation',
)
AUDIO_HEX_CLN_REMOTE = download_utils.RemoteFileMetadata(
    filename='audio_hex-pickup_debleeded.zip',
    url='https://zenodo.org/record/3371780/files/audio_hex-pickup_debleeded.zip?download=1',
    checksum='c31d97279464c9a67e640cb9061fb0c6',
    destination_dir='audio_hex-pickup_debleeded',
)
AUDIO_HEX_REMOTE = download_utils.RemoteFileMetadata(
    filename='audio_hex-pickup_original.zip',
    url='https://zenodo.org/record/3371780/files/audio_hex-pickup_original.zip?download=1',
    checksum='f9911bf217cb40e9e68edf3726ef86cc',
    destination_dir='audio_hex-pickup_original',
)
AUDIO_MIC_REMOTE = download_utils.RemoteFileMetadata(
    filename='audio_mono-mic.zip',
    url='https://zenodo.org/record/3371780/files/audio_mono-mic.zip?download=1',
    checksum='275966d6610ac34999b58426beb119c3',
    destination_dir='audio_mono-mic',
)
AUDIO_MIX_REMOTE = download_utils.RemoteFileMetadata(
    filename='audio_mono-pickup_mix.zip',
    url='https://zenodo.org/record/3371780/files/audio_mono-pickup_mix.zip?download=1',
    checksum='aecce79f425a44e2055e46f680e10f6a',
    destination_dir='audio_mono-pickup_mix',
)
_STYLE_DICT = {
    'Jazz': 'Jazz',
    'BN': 'Bossa Nova',
    'Rock': 'Rock',
    'SS': 'Singer-Songwriter',
    'Funk': 'Funk',
}
_GUITAR_STRINGS = ['E', 'A', 'D', 'G', 'B', 'e']
DATA = utils.LargeData('guitarset_index.json')


class Track(object):
    """GuitarSet track class

    Args:
        track_id (str): track id of the track
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets/GuitarSet`

    Attributes:
        track_id (str): track id
        audio_hex_cln_path (str): path to the debleeded hex wave file
        audio_hex_path (str): path to the original hex wave file
        audio_mic_path (str): path to the mono wave via microphone
        audio_mix_path (str): path to the mono wave via downmixing hex pickup
        jams_path (str): path to the jams file
        player_id (str):
            ID of the different players.
            one of ['00', '01', ... , '05']
        tempo (float): BPM of the track
        mode (str):
            one of ['solo', 'comp']
            For each excerpt, players are asked to first play in 'comp' mode
            and later play a 'solo' version on top of the already recorded comp.
        style (str):
            one of ['Jazz', 'Bossa Nova', 'Rock', 'Singer-Songwriter', 'Funk']
        beats (BeatData)
        leadsheet_chords (ChordData)
        inferred_chords (ChordData)
        key_mode (KeyData)
        pitch_contours (dict):
            {
                'E': F0Data(...),
                'A': F0Data(...),
                ...
                'e': F0Data(...)
            }
            a dict that contains 6 `F0Data`s.
            From Low E string to high e string.
        notes (list): (dict):
            {
                'E': NoteData(...),
                'A': NoteData(...),
                ...
                'e': NoteData(...)
            }
            a dict that contains 6 `NoteData`s.
            From Low E string to high e string.
        audio_mic (tuple): (np.ndarray, sr)
        audio_mix (tuple): (np.ndarray, sr)
        audio_hex (tuple): (np.ndarray, sr)
        audio_hex_cln (tuple): (np.ndarray, sr)
    """

    def __init__(self, track_id, data_home=None):
        if track_id not in DATA.index:
            raise ValueError('{} is not a valid track ID in GuitarSet'.format(track_id))

        self.track_id = track_id

        if data_home is None:
            data_home = utils.get_default_dataset_path(DATASET_DIR)

        self._data_home = data_home
        self._track_paths = DATA.index[track_id]

        self.audio_hex_cln_path = os.path.join(
            self._data_home, self._track_paths['audio_hex_cln'][0]
        )
        self.audio_hex_path = os.path.join(
            self._data_home, self._track_paths['audio_hex'][0]
        )
        self.audio_mic_path = os.path.join(
            self._data_home, self._track_paths['audio_mic'][0]
        )
        self.audio_mix_path = os.path.join(
            self._data_home, self._track_paths['audio_mix'][0]
        )
        self.jams_path = os.path.join(self._data_home, self._track_paths['jams'][0])

        title_list = track_id.split('_')  # [PID, S-T-K, mode, rec_mode]
        style, tempo, _ = title_list[1].split('-')  # [style, tempo, key]
        self.player_id = title_list[0]
        self.mode = title_list[2]
        self.tempo = float(tempo)
        self.style = _STYLE_DICT[style[:-1]]

    def __repr__(self):
        repr_string = (
            'GuitarSet Track('
            + 'track_id={}, jams_path={},\n'.format(self.track_id, self.jams_path)
            + 'tempo={}, mode={}, style={},\n'.format(self.tempo, self.mode, self.style)
            + "beats=BeatData('beat_times', 'beat_positions'),\n"
            + "leadsheet_chords=ChordData('start_times', 'end_times', 'chords'),\n"
            + "inferred_chords=ChordData('start_times', 'end_times', 'chords'),\n"
            + "key_mode=KeyData('start_times', 'end_times', 'keys'),\n"
            + "pitch_contours=dict(F0Data('times', 'frequencies', 'confidence')),\n"
            + "notes=dict(NoteData('start_times', 'end_times', 'notes', 'confidence')))"
        )
        return repr_string

    @utils.cached_property
    def beats(self):
        return _load_beats(self.jams_path)

    @utils.cached_property
    def leadsheet_chords(self):
        if self.mode == 'solo':
            logging.info(
                'Chord annotations for solo excerpts are the same with the comp excerpt.'
            )
        return _load_chords(self.jams_path, leadsheet_version=True)

    @utils.cached_property
    def inferred_chords(self):
        if self.mode == 'solo':
            logging.info(
                'Chord annotations for solo excerpts are the same with the comp excerpt.'
            )
        return _load_chords(self.jams_path, leadsheet_version=False)

    @utils.cached_property
    def key_mode(self):
        return _load_key_mode(self.jams_path)

    @utils.cached_property
    def pitch_contours(self):
        contours = {}
        # iterate over 6 strings
        for i in range(6):
            contours[_GUITAR_STRINGS[i]] = _load_pitch_contour(self.jams_path, i)
        return contours

    @utils.cached_property
    def notes(self):
        notes = {}
        # iterate over 6 strings
        for i in range(6):
            notes[_GUITAR_STRINGS[i]] = _load_note_ann(self.jams_path, i)
        return notes

    @property
    def audio_mic(self):
        """Load the audio for the 'mic' version of the GuitarSet Track.
        """
        audio, sr = librosa.load(self.audio_mic_path, sr=None)
        return audio, sr

    @property
    def audio_mix(self):
        """Load the audio for the 'mix' version of the GuitarSet Track.
        """
        audio, sr = librosa.load(self.audio_mix_path, sr=None)
        return audio, sr

    @property
    def audio_hex(self):
        """Load the audio for the 'hex' version of the GuitarSet Track.
        """
        audio, sr = librosa.load(self.audio_hex_path, sr=None, mono=False)
        return audio, sr

    @property
    def audio_hex_cln(self):
        """Load the audio for the 'hex_cln' version of the GuitarSet Track.
        """
        audio, sr = librosa.load(self.audio_hex_cln_path, sr=None, mono=False)
        return audio, sr


def download(data_home=None):
    """Download GuitarSet.

    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`
    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    download_utils.downloader(
        data_home,
        zip_downloads=[
            ANNOTATION_REMOTE,
            AUDIO_HEX_CLN_REMOTE,
            AUDIO_HEX_REMOTE,
            AUDIO_MIC_REMOTE,
            AUDIO_MIX_REMOTE,
        ],
        cleanup=True,
    )


def validate(data_home=None, silence=False):
    """Validate if the stored dataset is a valid version
    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`
    Returns:
        missing_files (list): List of file paths that are in the dataset index
            but missing locally
        invalid_checksums (list): List of file paths that file exists in the dataset
            index but has a different checksum compare to the reference checksum
    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    missing_files, invalid_checksums = utils.validator(
        DATA.index, data_home, silence=silence
    )
    return missing_files, invalid_checksums


def track_ids():
    """Return track ids
    Returns:
        (list): A list of track ids
    """
    return list(DATA.index.keys())


def load(data_home=None):
    """Load GuitarSet
    Args:
        data_home (str): Local path where GuitarSet is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`
    Returns:
        (dict): {`track_id`: track data}
    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    guitarset_data = {}
    for key in DATA.index.keys():
        guitarset_data[key] = Track(key, data_home=data_home)
    return guitarset_data


def _load_beats(jams_path):
    jam = jams.load(jams_path)
    anno = jam.search(namespace='beat_position')[0]
    times, values = anno.to_event_values()
    positions = [int(v['position']) for v in values]
    return utils.BeatData(times, positions)


def _load_chords(jams_path, leadsheet_version=True):
    """
    Parameters:
    -----------
    jams_path : str
        path of the jams annotation file
    leadsheet_version : Bool
        Whether or not to load the leadsheet version of the chord annotation
        If False, load the infered version.
    """
    jam = jams.load(jams_path)
    if leadsheet_version:
        anno = jam.search(namespace='chord')[0]
    else:
        anno = jam.search(namespace='chord')[1]
    intervals, values = anno.to_interval_values()
    return utils.ChordData(intervals[:, 0], intervals[:, 1], values)


def _load_key_mode(jams_path):
    jam = jams.load(jams_path)
    anno = jam.search(namespace='key_mode')[0]
    intervals, values = anno.to_interval_values()
    return utils.KeyData(intervals[:, 0], intervals[:, 1], values)


def _load_pitch_contour(jams_path, string_num):
    '''
    Parameters:
    -----------
    jams_path : str
        path of the jams annotation file
    string_num : int, in range(6)
        Which string to load.
        0 being the Low E string, 5 is the high e string.
    '''
    jam = jams.load(jams_path)
    anno_arr = jam.search(namespace='pitch_contour')
    anno = anno_arr.search(data_source=str(string_num))[0]
    times, values = anno.to_event_values()
    frequencies = [v['frequency'] for v in values]
    return utils.F0Data(times, frequencies, np.ones_like(times))


def _load_note_ann(jams_path, string_num):
    '''
    Parameters:
    -----------
    jams_path : str
        path of the jams annotation file
    string_num : int, in range(6)
        Which string to load.
        0 being the Low E string, 5 is the high e string.
    '''
    jam = jams.load(jams_path)
    anno_arr = jam.search(namespace='note_midi')
    anno = anno_arr.search(data_source=str(string_num))[0]
    intervals, values = anno.to_interval_values()
    return utils.NoteData(
        intervals[:, 0], intervals[:, 1], values, np.ones_like(values)
    )


def cite():
    """Print the reference"""

    cite_data = """
=========== MLA ===========
Xi, Qingyang, et al.
"GuitarSet: A Dataset for Guitar Transcription."
In Proceedings of the 19th International Society for Music Information Retrieval Conference (ISMIR). 2018.
========== Bibtex ==========
@inproceedings{xi2018guitarset,
    title={GuitarSet: A Dataset for Guitar Transcription},
    author={Xi, Qingyang and Bittner, Rachel M and Ye, Xuzhou and Pauwels, Johan and Bello, Juan P},
    booktitle={International Society of Music Information Retrieval (ISMIR)},
    year={2018}
}
"""
    print(cite_data)



from __future__ import absolute_import

import os
import pytest

from mirdata import rwc_jazz, utils
from tests.test_utils import DEFAULT_DATA_HOME
from tests.test_download_utils import mock_downloader


def test_track():
    # test data home None
    track_default = rwc_jazz.Track('RM-J004')
    assert track_default._data_home == os.path.join(DEFAULT_DATA_HOME, 'RWC-Jazz')

    # test data_home where the test data lives
    data_home = 'tests/resources/mir_datasets/RWC-Jazz'

    with pytest.raises(ValueError):
        rwc_jazz.Track('asdfasdf', data_home=data_home)

    track = rwc_jazz.Track('RM-J004', data_home=data_home)

    # test attributes are loaded as expected
    assert track.track_id == 'RM-J004'
    assert track._data_home == data_home
    assert track._track_paths == {
        'audio': ['audio/rwc-j-m01/4.wav', '7887ad17b7e4dcad9aa4605482e36cfa'],
        'sections': [
            'annotations/AIST.RWC-MDB-J-2001.CHORUS/RM-J004.CHORUS.TXT',
            '59cd67199cce9da16283b85338e5a9af',
        ],
        'beats': [
            'annotations/AIST.RWC-MDB-J-2001.BEAT/RM-J004.BEAT.TXT',
            'f3159206ae2f0aa86901248148f4021f',
        ],
    }
    assert (
        track.audio_path
        == 'tests/resources/mir_datasets/RWC-Jazz/' + 'audio/rwc-j-m01/4.wav'
    )
    assert track.piece_number == 'No. 4'
    assert track.suffix == 'M01'
    assert track.track_number == 'Tr. 04'
    assert track.title == 'Crescent Serenade (Piano Solo)'
    assert track.artist == 'Makoto Nakamura'
    assert track.duration_sec == '02:47'
    assert track.variation == 'Instrumentation 1'
    assert track.instruments == 'Pf'

    # test that cached properties don't fail and have the expected type
    assert type(track.sections) is utils.SectionData
    assert type(track.beats) is utils.BeatData

    # test audio loading functions
    y, sr = track.audio
    assert sr == 44100
    assert y.shape == (44100 * 2,)

    repr_string = (
        "RWC-Jazz Track(track_id=RM-J004, "
        + "audio_path=tests/resources/mir_datasets/RWC-Jazz/audio/rwc-j-m01/4.wav, "
        + "piece_number=No. 4, suffix=M01, track_number=Tr. 04, "
        + "title=Crescent Serenade (Piano Solo), artist=Makoto Nakamura, "
        + "duration_sec=02:47, variation=Instrumentation 1, instruments=Pf, "
        + "sections=SectionData('start_times', 'end_times', 'sections'), "
        + "beats=BeatData('beat_times', 'beat_positions'))"
    )
    assert track.__repr__() == repr_string


def test_track_ids():
    track_ids = rwc_jazz.track_ids()
    assert type(track_ids) is list
    assert len(track_ids) == 50


def test_load():
    data_home = 'tests/resources/mir_datasets/RWC-Jazz'
    rwc_jazz_data = rwc_jazz.load(data_home=data_home)
    assert type(rwc_jazz_data) is dict
    assert len(rwc_jazz_data.keys()) == 50

    rwc_jazz_data_default = rwc_jazz.load()
    assert type(rwc_jazz_data_default) is dict
    assert len(rwc_jazz_data_default.keys()) == 50


def test_load_metadata():
    data_home = 'tests/resources/mir_datasets/RWC-Jazz'
    metadata = rwc_jazz._load_metadata(data_home)
    assert metadata['data_home'] == data_home
    assert metadata['RM-J004'] == {
        'piece_number': 'No. 4',
        'suffix': 'M01',
        'track_number': 'Tr. 04',
        'title': 'Crescent Serenade (Piano Solo)',
        'artist': 'Makoto Nakamura',
        'duration_sec': '02:47',
        'variation': 'Instrumentation 1',
        'instruments': 'Pf',
    }

    metadata_none = rwc_jazz._load_metadata('asdf/asdf')
    assert metadata_none is None


def test_download(mock_downloader):
    rwc_jazz.download()
    mock_downloader.assert_called()


def test_validate():
    rwc_jazz.validate()
    rwc_jazz.validate(silence=True)


def test_cite():
    rwc_jazz.cite()



from __future__ import absolute_import

import numpy as np
import os

import pytest

from mirdata import medley_solos_db, utils
from tests.test_utils import DEFAULT_DATA_HOME
from tests.test_download_utils import mock_downloader


def test_track():
    # test data home None
    track_default = medley_solos_db.Track('d07b1fc0-567d-52c2-fef4-239f31c9d40e')
    assert track_default._data_home == os.path.join(
        DEFAULT_DATA_HOME, 'Medley-solos-DB'
    )

    data_home = 'tests/resources/mir_datasets/Medley-solos-DB'

    with pytest.raises(ValueError):
        medley_solos_db.Track('asdfasdf', data_home=data_home)

    track = medley_solos_db.Track(
        'd07b1fc0-567d-52c2-fef4-239f31c9d40e', data_home=data_home
    )

    assert track.track_id == 'd07b1fc0-567d-52c2-fef4-239f31c9d40e'
    assert track._data_home == data_home
    assert track._track_paths == {
        'audio': [
            'audio/Medley-solos-DB_validation-3_d07b1fc0-567d-52c2-fef4-239f31c9d40e.wav',
            '53ed28731399b67425775be598b50d1c',
        ]
    }
    assert (
        track.audio_path
        == 'tests/resources/mir_datasets/Medley-solos-DB/audio/'
        + 'Medley-solos-DB_validation-3_d07b1fc0-567d-52c2-fef4-239f31c9d40e.wav'
    )
    assert track.instrument == 'flute'
    assert track.instrument_id == 3
    assert track.song_id == 210
    assert track.subset == "validation"

    y, sr = track.audio
    assert y.shape == (65536,)
    assert sr == 22050

    repr_string = (
        'Medley-solos-DB Track(track_id=d07b1fc0-567d-52c2-fef4-239f31c9d40e, '
        + 'audio_path=tests/resources/mir_datasets/Medley-solos-DB/audio/'
        + 'Medley-solos-DB_validation-3_d07b1fc0-567d-52c2-fef4-239f31c9d40e.wav, '
        + 'instrument=flute, song_id=210, subset=validation)'
    )
    assert track.__repr__() == repr_string


def test_track_ids():
    track_ids = medley_solos_db.track_ids()
    assert type(track_ids) is list
    assert len(track_ids) == 21571


def test_load():
    data_home = 'tests/resources/mir_datasets/Medley-solos-DB'
    msdb_data = medley_solos_db.load(data_home=data_home)
    assert type(msdb_data) is dict
    assert len(msdb_data.keys()) == 21571

    msdb_data = medley_solos_db.load()
    assert type(msdb_data) is dict
    assert len(msdb_data.keys()) == 21571


def test_download(mock_downloader):
    medley_solos_db.download()
    mock_downloader.assert_called()


def test_validate():
    medley_solos_db.validate()
    medley_solos_db.validate(silence=True)


def test_cite():
    cite_str = medley_solos_db.cite()



from __future__ import absolute_import

import numpy as np
import os
import json

import pytest

from mirdata import medleydb_melody, utils
from tests.test_utils import mock_validated, mock_validator, DEFAULT_DATA_HOME
from tests.test_download_utils import mock_downloader


def test_track():
    # test data home None
    track_default = medleydb_melody.Track('MusicDelta_Beethoven')
    assert track_default._data_home == os.path.join(
        DEFAULT_DATA_HOME, 'MedleyDB-Melody'
    )

    data_home = 'tests/resources/mir_datasets/MedleyDB-Melody'

    with pytest.raises(ValueError):
        medleydb_melody.Track('asdfasdf', data_home=data_home)

    track = medleydb_melody.Track('MusicDelta_Beethoven', data_home=data_home)

    # test attributes
    assert track.track_id == 'MusicDelta_Beethoven'
    assert track._data_home == data_home
    assert track._track_paths == {
        'audio': [
            'audio/MusicDelta_Beethoven_MIX.wav',
            '4c6081420a506b438a851c2807fc28ea',
        ],
        'melody1': [
            'melody1/MusicDelta_Beethoven_MELODY1.csv',
            '67dca3f4a9bf0517dd8a1287d091791e',
        ],
        'melody2': [
            'melody2/MusicDelta_Beethoven_MELODY2.csv',
            '67dca3f4a9bf0517dd8a1287d091791e',
        ],
        'melody3': [
            'melody3/MusicDelta_Beethoven_MELODY3.csv',
            '340f647c4f12d7e1ecf2421d0dfd509f',
        ],
    }
    assert (
        track.audio_path
        == 'tests/resources/mir_datasets/'
        + 'MedleyDB-Melody/audio/MusicDelta_Beethoven_MIX.wav'
    )
    assert track.artist == 'MusicDelta'
    assert track.title == 'Beethoven'
    assert track.genre == 'Classical'
    assert track.is_excerpt is True
    assert track.is_instrumental is True
    assert track.n_sources == 18

    assert type(track.melody1) is utils.F0Data
    assert type(track.melody2) is utils.F0Data
    assert type(track.melody3) is utils.F0Data

    y, sr = track.audio
    assert sr == 44100
    assert y.shape == (44100 * 2,)

    repr_string = (
        "MedleyDb-Melody Track(track_id=MusicDelta_Beethoven, "
        + "audio_path=tests/resources/mir_datasets/MedleyDB-Melody/audio/"
        + "MusicDelta_Beethoven_MIX.wav, artist=MusicDelta, title=Beethoven,"
        + " genre=Classical, is_excerpt=True, is_instrumental=True, "
        + "n_sources=18, melody1=F0Data('times', 'frequencies', confidence'),"
        + " melody2=F0Data('times', 'frequencies', confidence'), "
        + "melody3=F0Data('times', 'frequencies', confidence'))"
    )
    assert track.__repr__() == repr_string


def test_track_ids():
    track_ids = medleydb_melody.track_ids()
    assert type(track_ids) is list
    assert len(track_ids) == 108


def test_load():
    data_home = 'tests/resources/mir_datasets/MedleyDB-Melody'
    medleydb_melody_data = medleydb_melody.load(data_home=data_home)
    assert type(medleydb_melody_data) is dict
    assert len(medleydb_melody_data.keys()) is 108

    medleydb_melody_data_default = medleydb_melody.load()
    assert type(medleydb_melody_data_default) is dict
    assert len(medleydb_melody_data_default.keys()) is 108


def test_load_melody():
    # load a file which exists
    melody_path = (
        'tests/resources/mir_datasets/MedleyDB-Melody/'
        + 'melody1/MusicDelta_Beethoven_MELODY1.csv'
    )
    melody_data = medleydb_melody._load_melody(melody_path)

    # check types
    assert type(melody_data) == utils.F0Data
    assert type(melody_data.times) is np.ndarray
    assert type(melody_data.frequencies) is np.ndarray
    assert type(melody_data.confidence) is np.ndarray

    # check values
    assert np.array_equal(
        melody_data.times, np.array([0.0058049886621315194, 0.052244897959183675])
    )
    assert np.array_equal(melody_data.frequencies, np.array([0.0, 965.99199999999996]))
    assert np.array_equal(melody_data.confidence, np.array([0.0, 1.0]))

    # load a file which doesn't exist
    melody_data_none = medleydb_melody._load_melody('fake/file/path')
    assert melody_data_none is None


def test_load_melody3():
    # load a file which exists
    melody_path = (
        'tests/resources/mir_datasets/MedleyDB-Melody/'
        + 'melody3/MusicDelta_Beethoven_MELODY3.csv'
    )
    melody_data = medleydb_melody._load_melody3(melody_path)

    # check types
    assert type(melody_data) == utils.F0Data
    assert type(melody_data.times) is np.ndarray
    assert type(melody_data.frequencies) is np.ndarray
    assert type(melody_data.confidence) is np.ndarray

    # check values
    assert np.array_equal(
        melody_data.times,
        np.array([0.046439909297052155, 0.052244897959183675, 0.1219047619047619]),
    )
    assert np.array_equal(
        melody_data.frequencies,
        np.array(
            [
                [0.0, 0.0, 497.01600000000002, 0.0, 0.0],
                [965.99199999999996, 996.46799999999996, 497.10599999999999, 0.0, 0.0],
                [
                    987.32000000000005,
                    987.93200000000002,
                    495.46800000000002,
                    495.29899999999998,
                    242.98699999999999,
                ],
            ]
        ),
    )
    assert np.array_equal(
        melody_data.confidence,
        np.array(
            [
                [0.0, 0.0, 1.0, 0.0, 0.0],
                [1.0, 1.0, 1.0, 0.0, 0.0],
                [1.0, 1.0, 1.0, 1.0, 1.0],
            ]
        ),
    )

    # load a file which doesn't exist
    melody_data_none = medleydb_melody._load_melody3('fake/file/path')
    assert melody_data_none is None


def test_load_metadata():
    data_home = 'tests/resources/mir_datasets/MedleyDB-Melody'
    metadata = medleydb_melody._load_metadata(data_home)
    assert metadata['data_home'] == data_home
    assert metadata['MusicDelta_Beethoven'] == {
        'audio_path': 'MedleyDB-Melody/audio/MusicDelta_Beethoven_MIX.wav',
        'melody1_path': 'MedleyDB-Melody/melody1/MusicDelta_Beethoven_MELODY1.csv',
        'melody2_path': 'MedleyDB-Melody/melody2/MusicDelta_Beethoven_MELODY2.csv',
        'melody3_path': 'MedleyDB-Melody/melody3/MusicDelta_Beethoven_MELODY3.csv',
        'artist': 'MusicDelta',
        'title': 'Beethoven',
        'genre': 'Classical',
        'is_excerpt': True,
        'is_instrumental': True,
        'n_sources': 18,
    }

    metadata_none = medleydb_melody._load_metadata('asdf/asdf')
    assert metadata_none is None


def test_download(mock_downloader):
    medleydb_melody.download()
    mock_downloader.assert_called()


def test_validate():
    medleydb_melody.validate()
    medleydb_melody.validate(silence=True)


def test_cite():
    medleydb_melody.cite()
